import { split_promise } from "../lib.ts";

export class Event<V> {
	subscriber = [] as ((value: V) => unknown)[];

	subscribe(callback: (value: V) => unknown) {
		this.subscriber.push(callback);
	}

	unsubscribe(callback: (value: V) => unknown) {
		const index = this.subscriber.indexOf(callback);
		if (index === -1) return false;
		this.subscriber.splice(index, 1);
		return true;
	}

	trigger(value: V) {
		for (const handler of this.subscriber) handler(value);
	}

	async wait() {
		const [promise, resolver] = split_promise<V>();
		this.subscribe(resolver);
		await promise;
		this.unsubscribe(resolver);
	}
}
